(function (hui) {
    if(hui.indexedDB) return;
	var vendor = hui.getVendor(),
		indexedDB  = window.indexedDB || window[vendor + 'IndexedDB'];
	if(!indexedDB) return;
	hui.indexedDB = {
		/*
		 * @作者：416580612@qq.com
		 * @时间：2013/06/20 12:07:03
		 * @param options:
		 * @描述：
		 * name: database name
		 * version: database version
		 * onerror : 
		 * onsuccess :
		 * onupgradeneeded
		 */
		init: function(options){
			
			var request = indexedDB.open(options.name, options.version || 1);
			request.onerror = options.onerror;
			
			request.onsuccess = function(e){
				hui.indexedDB.db = indexedDB.db = e.target.result;
				var s = request;
				if(options.onsuccess)options.onsuccess(e);
			};
			request.onupgradeneeded = options.onupgradeneeded;
			request.onblocked = function(){
				//alert('database blocked');
			}
		},
		/*
		 * @作者：416580612@qq.com
		 * @时间：2013/06/20 12:13:59
		 * @param options:
		 * @描述：
		 * name: store name,
		 * data: data to store,
		 * onsuccess:
		 * onerror:
		 */
		add: function(options){
			try{
				var trans, store = options.store;
				if(!store){
					trans = indexedDB.db.transaction(options.name, "readwrite");
					store = trans.objectStore(options.name);
				}
				if(trans){
					trans.oncomplete = function(e){
						if(options.onsuccess)options.onsuccess(e, options)
					}
				}
				var req = store.add(options.data);

				req.onsuccess = function(e){
					if(!trans && options.onsuccess) options.onsuccess(e, options);
				}
				req.onerror = function(e){
					if(options.onerror) options.onerror(e, options);
				}
			}
			catch(ex){
				hui.showErr(ex);
				throw ex;
			}
		},
		/*
		 * @作者：416580612@qq.com
		 * @时间：2013/06/20 12:13:59
		 * @param options:
		 * @描述：
		 * name: store name,
		 * data: data to store,
		 * onsuccess:
		 * onerror:
		 * oncomplete
		 */
		put: function(options){
			try{
				var trans, store = options.store;
				if(!store){
					//alert('put 0');
					trans = indexedDB.db.transaction([options.name], "readwrite");
					//alert('put 1');
					store = trans.objectStore(options.name);
				}
				if(trans){
					trans.oncomplete = function(e){
						if(options.onsuccess)options.onsuccess(e, options)
					}
				}
				var req = store.put(options.data);

				req.onsuccess = function(e){
					if(!trans && options.onsuccess) options.onsuccess(e, options);
				}
				req.onerror = function(e){
					if(options.onerror) options.onerror(e, options);
				}
			}
			catch(ex){
				hui.showErr(ex, 'put' + JSON.stringify(options));
				throw ex;
			}
		},
		/*
		 * @作者：416580612@qq.com
		 * @时间：2013/06/20 12:13:59
		 * @param options:
		 * @描述：
		 * name: store name,
		 * data: data to store,
		 * onsuccess:
		 * onerror:
		 * oncomplete
		 */
		get: function(options){
			try{
				var trans, store = options.store;
				if(!store){
					trans = indexedDB.db.transaction([options.name], "readonly");
					store = trans.objectStore(options.name);
				}
				var req = store.get(options.keyvalue);
				req.onsuccess = function(e){
					if(options.onsuccess) options.onsuccess(e, options)
				};
				req.onerror = function(e){
					if(options.onerror) options.onerror(e, options)
				};
			}
			catch(ex){
				hui.showErr(ex + 'get');
				throw ex;
			}
		},
		/**
		 * @作者：416580612@qq.com
		 * @时间：2013/06/20 12:27:01
		 * @描述： 
		 * name: store name,
		 * key: key name
		 * pageIndex: page index
		 * pageCount: page count,
		 * onsuccess:
		 * onerror
		 */
		getList: function(options){
			try{
				var trans = indexedDB.db.transaction([options.name], "readonly");
					store = trans.objectStore(options.name),
					req = store.index(options.key).openCursor(options.keyRange || null, options.direction||'prev'),
					pageIndex = options.pageIndex,
					pageCount = options.pageCount,
					hasAdvance = !pageIndex || !pageCount || pageIndex < 2 ? true: false,
					data = [],
					index=0;
				req.onsuccess = function(e){
					var cursor = e.target.result;
					if(!hasAdvance){
						hasAdvance = true;
						cursor.advance((pageIndex - 1) *pageCount);
					}
					else if(cursor){
						data.push(cursor.value);
						index++;
						if(pageCount && pageIndex && index >= pageCount) {
							if(options.onsuccess) options.onsuccess(data);
							return;
						}
						cursor.continue();
					}else{
						if(options.onsuccess) options.onsuccess(data)
					}
				}
				req.onerror = function(e){
					if(options.onerror) options.onerror(e);
				}
			}
			catch(ex){
				hui.showErr(ex, 'getList');
				throw ex;
			}
		}
	}
})(hui);